# -*- coding: utf-8 -*-

import queue
from time import monotonic as time


class SetQueue(queue.Queue):
    def __init__(self):
        queue.Queue.__init__(self)

    def _put(self, item):
        if item not in self.queue:
            queue.Queue._put(self, item)

    def peek(self, block=True, timeout=None):
        """Return an item from the queue.

        If optional args 'block' is true and 'timeout' is None (the default),
        block if necessary until an item is available. If 'timeout' is
        a non-negative number, it blocks at most 'timeout' seconds and raises
        the Empty exception if no item was available within that time.
        Otherwise ('block' is false), return an item if one is immediately
        available, else raise the Empty exception ('timeout' is ignored
        in that case).
        """
        with self.not_empty:
            if not block:
                if not self._qsize():
                    raise queue.Empty
            elif timeout is None:
                while not self._qsize():
                    self.not_empty.wait()
            elif timeout < 0:
                raise ValueError("'timeout' must be a non-negative number")
            else:
                endtime = time() + timeout
                while not self._qsize():
                    remaining = endtime - time()
                    if remaining <= 0.0:
                        raise queue.Empty
                    self.not_empty.wait(remaining)
            return self._peek()

    def _peek(self):
        return next(iter(self.queue))


class DictDiffer(object):
    """
    Calculate the difference between two dictionaries as:
    (1) items added
    (2) items removed
    (3) keys same in both but changed values
    (4) keys same in both and unchanged values
    """

    def __init__(self, current_dict, past_dict):
        self.current_dict, self.past_dict = current_dict, past_dict
        self.set_current, self.set_past = set(current_dict.keys()), set(past_dict.keys())
        self.intersect = self.set_current.intersection(self.set_past)

    def added(self):
        return self.set_current - self.intersect

    def removed(self):
        return self.set_past - self.intersect

    def changed(self):
        return set(o for o in self.intersect if self.past_dict[o] != self.current_dict[o])

    def unchanged(self):
        return set(o for o in self.intersect if self.past_dict[o] == self.current_dict[o])



# -*- coding: utf-8 -*-
import configparser
import datetime
import os
import shutil
import urllib.request
from html.parser import HTMLParser

import click
from jinja2 import TemplateNotFound

import blogbook.constant
import blogbook.toolbox.file


class HtmlStripper(HTMLParser):
    def __init__(self):
        HTMLParser.__init__(self)
        self.reset()
        self.strict = False
        self.convert_charrefs = True
        self.fed = []

    def error(self, message):
        pass

    def handle_data(self, d):
        self.fed.append(d)

    def get_data(self):
        return ''.join(self.fed)


def pre_process_dynamic(file_path, filename, output_dir, setting, markdown, base_path):
    relative_path = os.path.relpath(blogbook.toolbox.file.parentdir(file_path), base_path)
    base_url = blogbook.constant.URL_SEPARATOR
    if relative_path != os.path.curdir:
        base_url += urllib.request.pathname2url(relative_path) + blogbook.constant.URL_SEPARATOR

    # filename_pattern = blogbook.constant.FILENAME_REGEX.match(filename)
    # output_name = filename_pattern.group(5)
    name = filename.replace('.md', '')

    page = {
        'title': name.capitalize(),
        'url': base_url + name,
        'layout': setting['blog']['default_layout'],
        'content': {
            'path': file_path,
            'position': 0,
            'output_dir': output_dir,
            'output_path': os.path.join(output_dir, name)
        }
    }
    # if filename_pattern.group(2) and filename_pattern.group(3) and filename_pattern.group(4):
    #     year = int(filename_pattern.group(2))
    #     month = int(filename_pattern.group(3))
    #     day = int(filename_pattern.group(4))
    #     page['date'] = datetime.date(year, month, day)

    if os.path.isfile(file_path):

        with open(file_path, 'r') as stream:

            content_position = int(stream.tell())

            line = stream.readline()
            while line and not line.strip():
                content_position = int(stream.tell())
                line = stream.readline()

            if line.strip() == '---':

                meta = ''

                line = stream.readline()
                while line and not line.strip() == '---':
                    meta += line
                    line = stream.readline()
                    content_position = int(stream.tell())

                if not line.strip() == '---':
                    raise click.UsageError('Meta section open but never close on content file [{}]'.format(filename))

                configuration = configparser.ConfigParser()
                configuration.read_string(meta)
                if configuration.has_section('page'):
                    page.update(configuration['page'])

                line = stream.readline()
                while line and not line.strip():
                    content_position = int(stream.tell())
                    line = stream.readline()

            if 'excerpt' not in page:

                excerpt = line

                line = stream.readline()
                while line and not line == setting['excerpt']['separator']:
                    excerpt += line
                    line = stream.readline()

                if excerpt and excerpt.strip():

                    stripper = HtmlStripper()
                    stripper.feed(markdown(excerpt))
                    page['excerpt'] = stripper.get_data()

                    excerpt_words = page['excerpt'].split()
                    if len(excerpt_words) > setting['excerpt']['max_word']:
                        page['excerpt'] = ' '.join(excerpt_words[:setting['excerpt']['max_word']]) + '...'

        page['content']['position'] = content_position

    return page


def process_static(file_path, output_dir):
    if not os.path.isdir(output_dir):
        os.makedirs(output_dir, exist_ok=True)

    shutil.copy(file_path, output_dir)


def pre_process(path, output_dir, context, markdown, base_content_path=None):
    """
    Scan source directory in order to :
        - Copy any file other than content file (*.md) into destination directory.
        - Parse content file to populate site dictionary context.
    """
    base_content_path = base_content_path if base_content_path else path

    for entry in os.scandir(path):

        if not any(
            content_exclude_pattern.match(entry.name) is not None
            for content_exclude_pattern in context['setting']['path']['content_exclude_patterns']
        ):

            if entry.is_dir():
                pre_process(
                    entry.path,
                    os.path.join(output_dir, entry.name),
                    context,
                    markdown,
                    base_content_path
                )

            elif entry.is_file():

                if entry.name.lower().endswith(blogbook.constant.MARKDOWN_EXT):

                    page = pre_process_dynamic(
                        entry.path, entry.name, output_dir, context['setting'], markdown, base_content_path
                    )

                    layout_group = page['layout'] + 's'
                    if layout_group in context['site']:
                        context['site'][layout_group].append(page)
                    else:
                        context['site'][layout_group] = [page]

                else:
                    process_static(entry.path, output_dir)


def post_process_dynamic(page, context, markdown, environment):
    """
    Scan site dictionary context in order to generate static html page
    base on site setting, template and page content.
    """

    output_dir = page['content']['output_dir']
    if not os.path.isdir(output_dir):
        os.makedirs(output_dir, exist_ok=True)

    with open(page['content']['output_path'], 'w') as output_stream:

        context['page'] = page

        if 'path' in page['content']:
            with open(page['content']['path'], 'r') as source_stream:
                source_stream.seek(page['content']['position'])
                context['content'] = markdown(source_stream.read())

        try:
            output_stream.write(
                environment.get_template(page['layout'] + blogbook.constant.LAYOUT_EXT).render(context)
            )
        except TemplateNotFound as ex:
            raise click.UsageError('Missing required layout : {}'.format(ex.message))

        context.pop('content', None)



# -*- coding: utf-8 -*-
import configparser
import getpass
import os
import re
import shutil
import sys
import zipfile

import click
import mistune
import paramiko
from PIL import Image
from jinja2 import Environment, FileSystemLoader
from dulwich import porcelain
from dulwich.repo import Repo

import blogbook.build.process
import blogbook.build.watch
import blogbook.constant
import blogbook.repo.push
import blogbook.serve
import blogbook.toolbox.file


class Blogbook(object):
    def __init__(self, home):
        self.home = home
        self.verbose = False
        self.setting = {
            'path': {
                'content_exclude_patterns': ['_*', '.*'],
                'layouts': os.path.join(home, blogbook.constant.LAYOUTS_DIRECTORY),
                'output': os.path.join(home, blogbook.constant.DEFAULT_OUTPUT)
            },
            'blog': {
                'default_layout': 'post',
                'language': 'en',
                'name': os.path.basename(home).capitalize(),
            },
            'excerpt': {
                'separator': '\n',
                'max_word': 20
            },
            'author': {
                'name': getpass.getuser()
            }
        }

        self.setting_path = os.path.join(home, blogbook.constant.LOGBOOK_DIRECTORY, 'config')
        if os.path.isfile(self.setting_path):
            configuration = configparser.ConfigParser()
            configuration.read(self.setting_path)
            self.setting.update(configuration._sections)

        self.setting['path']['content_exclude_patterns'] = [
            re.compile(content_exclude_pattern.replace('.', '\.').replace('*', '.*'))
            for content_exclude_pattern in self.setting['path']['content_exclude_patterns']
        ]

    def set_config(self, key, value):
        self.setting[key] = value
        if self.verbose:
            click.echo('  config[%s] = %s'.format(key, value), file=sys.stderr)

    def __repr__(self):
        return '<Blogbook {}>'.format(self.home)


pass_blog = click.make_pass_decorator(Blogbook)


@click.group()
@click.option(
    '--home',
    '-h',
    default=os.getcwd()
)
@click.option(
    '--verbose',
    '-v',
    is_flag=True,
    help='Enables verbose mode.'
)
@click.version_option()
@click.pass_context
def main(context, home, verbose):
    """Repo is a command line tool that showcases how to build complex
    command line interfaces with Click.
    This tool is supposed to look like a distributed version control
    system to show how something like this can be structured.
    """
    context.obj = Blogbook(os.path.abspath(home))
    context.obj.verbose = verbose


@main.command()
@click.option(
    '--set',
    nargs=2,
    metavar='KEY VALUE',
    help='Overwrite a config key/value pair.'
)
@click.option(
    '--unset',
    metavar='KEY',
    help='Remove a config key and is value.'
)
@pass_blog
def config(blog, set, unset):
    """
    Configure a new blogbook.
    """

    if not os.path.isdir(os.path.join(blog.home, blogbook.constant.LOGBOOK_DIRECTORY)):
        raise click.UsageError('Not a blogbook directory : {}'.format(blog.home))

    configuration = configparser.ConfigParser()
    if os.path.isfile(blog.setting_path):
        configuration.read(blog.setting_path)

    if set:
        section, option = set[0].split('.', 1)
        if not configuration.has_section(section):
            configuration.add_section(section)
        configuration.set(section, option, set[1])
        with open(blog.setting_path, 'w') as stream:
            configuration.write(stream)
    if unset:
        section, option = unset.split('.', 1)
        if configuration.has_section(section):
            configuration.remove_option(section, option)
            if not configuration[section]:
                configuration.remove_section(section)
        with open(blog.setting_path, 'w') as stream:
            configuration.write(stream)


@main.command()
@click.option(
    '-f', '--force',
    is_flag=True,
    help='Force override existing directory'
)
@click.option(
    '-l', '--layout',
    default=os.path.join(blogbook.constant.BUILTIN_LAYOUT_PATH, 'default'),
    metavar='PATH',
    help='The blogbook template path.'
)
@pass_blog
def new(blog, force, layout):
    """
    Create a new blogbook.
    """

    if os.path.isdir(blog.home) and not blogbook.toolbox.file.is_empty(blog.home) and not force:
        raise click.UsageError(
            blog.home + ' already exist and is not an empty directory, '
            + 'either user an other directory or use the --force flag.'
        )

    if not os.path.isdir(layout) and not zipfile.is_zipfile(layout):
        raise click.BadParameter('You must provide either an existing layout directory or zip file')

    click.echo('Create new blogbook into [{}]'.format(blog.home))
    if not os.path.isdir(blog.home):
        os.makedirs(blog.home, exist_ok=True)

    logbook_work_path = os.path.join(blog.home, blogbook.constant.LOGBOOK_DIRECTORY)
    blogbook.toolbox.file.force_mkdir(logbook_work_path)
    with open(os.path.join(logbook_work_path, 'version'), 'w') as stream_logbook:
        stream_logbook.write('0')

    git_dir = os.path.join(blog.home, '.git')
    if os.path.isdir(git_dir):
        shutil.rmtree(git_dir)
    porcelain.init(blog.home)

    click.echo('with layouts [{}]'.format(os.path.basename(layout)))
    layouts_path = blog.setting['path']['layouts']
    if os.path.isdir(layouts_path):
        shutil.rmtree(layouts_path)

    if os.path.isdir(layout):
        shutil.copytree(layout, layouts_path)
    elif zipfile.is_zipfile(layout):
        with zipfile.ZipFile(layout) as zf:
            zf.extractall(layouts_path)


@main.command()
@click.option(
    '-w', '--watch',
    is_flag=True,
    help='Build on change and run server.'
)
@pass_blog
def build(blog, watch):
    """
    Build a blogbook from his template.
    """

    output = blog.setting['path']['output']
    layouts = blog.setting['path']['layouts']

    if not os.path.isdir(layouts) or blogbook.toolbox.file.is_empty(layouts):
        raise click.UsageError('Missing layout into ' + layouts)

    blogbook.toolbox.file.force_mkdir(output)

    assets_path = os.path.join(layouts, blogbook.constant.ASSETS_DIRECTORY)
    if os.path.isdir(assets_path) and not blogbook.toolbox.file.is_empty(assets_path):
        shutil.copytree(assets_path, os.path.join(output, blogbook.constant.ASSETS_DIRECTORY))

    context = {
        'setting': blog.setting,
        'site': {}
    }
    markdown = mistune.Markdown()

    blogbook.build.process.pre_process(blog.home, output, context, markdown)

    def get_key(site_page):
        return site_page.get('date') or '0001-01-01', site_page['title']

    for group, pages in context['site'].items():
        context['site'][group] = sorted(pages, reverse=True, key=get_key)

    environment = Environment(loader=FileSystemLoader(layouts))
    for pages in context['site'].values():
        for page in pages:
            blogbook.build.process.post_process_dynamic(page, context, markdown, environment)

    if watch:
        watcher = blogbook.build.watch.LogbookContentWatcher(blog.home, output, context, markdown, environment)
        try:
            blogbook.serve.run_server(blog.home, output, 1234, blog.setting)
        except KeyboardInterrupt:
            watcher.stop()


@main.command()
@click.option(
    '-p', '--port',
    default=1234,
    metavar='PORT',
    help='Port on which binding the server.'
)
@pass_blog
def serve(blog, port):
    """
    Serve a blogbook generated content for test purpose only.
    """
    blogbook.serve.run_server(blog.home, blog.setting['path']['output'], port, blog.setting)


@main.command()
@pass_blog
def push(blog):
    """
    Serve a blogbook generated content for test purpose only.
    """

    # if 'remote' not in blog.setting or 'host' not in blog.setting['remote']:
    #     raise click.UsageError('Missing remote host setting')

    r = Repo(blog.home)
    porcelain.add(r)

    s = porcelain.status(r)
    if any(c for c in s.staged.values()):

        porcelain.commit(r, b'Commit/Push blog', committer=b'Tui <tui@tui.ca>')
        # porcelain.pus

    # client = paramiko.SSHClient()
    # client.load_host_keys(blogbook.constant.SSH_HOST_KEYS)
    # client.set_missing_host_key_policy(blogbook.repo.push.ConfirmAddPolicy())
    #
    # client.connect(blog.setting['remote']['host'])
    #
    # remote_root = '/var/www/blogbook/' + os.path.basename(blog.home)
    # client.exec_command('rm -rf {}'.format(remote_root))
    # client.exec_command('mkdir -p {}'.format(remote_root))
    #
    # sftp = client.open_sftp()
    # for local_root, dir_names, file_names in os.walk(blog.home):
    #
    #     current_remote_root = remote_root
    #     relative_path = os.path.relpath(local_root, blog.home)
    #     if relative_path != os.curdir:
    #         current_remote_root += '/' + relative_path.replace('\\', '/')
    #
    #     for file_name in file_names:
    #         local_file_path = os.path.join(local_root, file_name)
    #         remote_file_path = current_remote_root + '/' + file_name
    #         click.echo('{} --> {}'.format(local_file_path, remote_file_path))
    #         sftp.put(local_file_path, remote_file_path)
    #     for dir_name in dir_names:
    #         remote_dir_path = current_remote_root + '/' + dir_name
    #         click.echo('{} --> {}'.format(os.path.join(local_root, dir_name), remote_dir_path))
    #         sftp.mkdir(remote_dir_path)
    #
    # client.close()


@main.command()
@click.argument('source')
@click.argument('destination')
@pass_blog
def add(blog, source, destination):
    if os.path.isfile(source) and source.lower().endswith('.jpg'):
        im = Image.open(source)
        im.thumbnail((800, 1200), Image.ANTIALIAS)
        im.save(os.path.join(blog.home, destination), optimize=True, quality=75)


if __name__ == "__main__":
    main()



from setuptools import setup, find_packages

setup(
    name='blogbook',
    version='0.10',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'click',
        'jinja2',
        'mistune',
        'pyyaml',
        'watchdog',
        'paramiko',
        'Pillow',
        'dulwich'
    ],
    entry_points={
        'console_scripts': ['blogbook=blogbook.command:main']
    },
    url='https://github.com/pieceofstone/blogbook',
    author='noogg',
    author_email='tkclem@yahoo.fr',
)



# -*- coding: utf-8 -*-

import os
import re

LOGBOOK_DIRECTORY = '.blogbook'

CONTENT_DIRECTORY = 'content'
FILENAME_REGEX = re.compile('^((\d{4})-(\d{2})-(\d{2})-)?(.*)\.md$', re.IGNORECASE)
MARKDOWN_EXT = '.md'

DEV_EDITOR_PATH = os.path.join(os.path.dirname(__file__), 'editor')

BUILTIN_LAYOUT_PATH = os.path.join(os.path.dirname(__file__), 'layouts')
LAYOUTS_DIRECTORY = '_layout'
LAYOUT_EXT = '.html'
ASSETS_DIRECTORY = 'assets'

URL_SEPARATOR = '/'

SSH_HOST_KEYS = os.path.expanduser('~/.ssh/known_hosts')

DEFAULT_OUTPUT = '_html'
